%bcond_without dpdk
%bcond_without libcapng
%bcond_without afxdp
%bcond_with check_datapath_kernel 

%ifarch x86_64
%bcond_without check
%else
%bcond_with check
%endif


Summary: Open vSwitch daemon/database/utilities
Name: openvswitch
Version: 3.1.1
Release: 5%{?dist}
License: ASL 2.0 and LGPLv2+ and SISSL
URL: https://www.openvswitch.org/
Source0: %{url}releases/%{name}-%{version}.tar.gz
Source1: openvswitch.sysusers

#upstream: https://github.com/openvswitch/ovs/commit/91e621bd5abab19954bec09c7d27c59acdf607b1
Patch0001: fix-CVE-2023-3966.patch

Patch3000: 0000-openvswitch-add-stack-protector-strong.patch
Patch3001: 0002-Remove-unsupported-permission-names.patch
Patch3002: fix-selinux-err.patch
# fix warnings in versions above groff-1.23 
Patch3003: openvswitch-tbl-warning-groff-1.23.patch
patch5000: openvswitch-troff-warning-groff-1.23.patch

BuildRequires: gcc gcc-c++ make autoconf automake libtool
BuildRequires: systemd-rpm-macros openssl openssl-devel
BuildRequires: python3-devel python3-six python3-setuptools python3-sortedcontainers
BuildRequires: python3-sphinx desktop-file-utils groff-base graphviz unbound-devel
BuildRequires: procps-ng groff python3-pyOpenSSL systemtap-sdt-devel
%if %{with afxdp}
BuildRequires: libxdp-devel libbpf-devel numactl-devel
%endif
%if %{with check_datapath_kernel}
BuildRequires: nmap-ncat
%endif 
%if %{with libcapng}
BuildRequires: libcap-ng libcap-ng-devel
%endif 
%if %{with dpdk}
BuildRequires: dpdk-devel libpcap-devel numactl-devel
BuildRequires: libibverbs >= 15
%endif

Requires: openssl iproute module-init-tools
%{?systemd_requires}
%{?sysusers_requires_compat}
Requires(post): /bin/sed
Requires(post): %{_sbindir}/update-alternatives
Requires(postun): %{_sbindir}/update-alternatives

%description
Open vSwitch is a production quality, multilayer virtual switch. It provides standard network 
bridging functions and support for the OpenFlow protocol for remote per-flow control of traffic.

%package -n python3-openvswitch
Summary: Open vSwitch python3 bindings
License: ASL 2.0
Requires: python3 python3-six
Provides: python-openvswitch = %{version}-%{release}

%description -n python3-openvswitch
This package provides python bindings for the Open vSwitch database.

%package test
Summary: Open vSwitch testing utilities
License: ASL 2.0
BuildArch: noarch
Requires: python3-openvswitch = %{version}-%{release}

%description test
This package provides utilities that are useful to diagnose performance and connectivity issues 
in Open vSwitch setup.

%package testcontroller
Summary: Simple controller for testing OpenFlow setups
License: ASL 2.0
Requires: openvswitch = %{version}-%{release}

%description testcontroller
This controller enables OpenFlow switches that connect to it to act as MAC-learning Ethernet 
switches. It can be used for initial testing of OpenFlow networks. It is not a necessary or 
desirable part of a production OpenFlow deployment.

%package devel
Summary: Open vSwitch OpenFlow development package (library, headers)
License: ASL 2.0

%description devel
This package provides shared library, libopenswitch.so and the openvswitch header files needed to build 
an external application.

%package -n network-scripts-%{name}
Summary: Open vSwitch legacy network service support
License: ASL 2.0
Requires: network-scripts
Supplements: (%{name} and network-scripts)
 
%description -n network-scripts-%{name}
This package provides the ifup and ifdown scripts for use with the legacy network service.

%package ipsec
Summary: Open vSwitch IPsec tunneling support
License: ASL 2.0
Requires: openvswitch libreswan
Requires: python3-openvswitch = %{version}-%{release}

%description ipsec
This package provides IPsec tunneling support for OVS tunnels.

%if %{with dpdk}
%package dpdk
Summary: Open vSwitch OpenFlow development package (switch, linked with DPDK)
License: ASL 2.0
Supplements: %{name}

%description dpdk
This package provides ovs-vswitchd linked with DPDK library.
%endif


%prep
%autosetup -p 1


%build
rm -f python/ovs/dirs.py
autoreconf

./boot.sh
mkdir build build-dpdk
pushd build
ln -s ../configure
%configure \
%if %{with libcapng}
        --enable-libcapng \
%else
        --disable-libcapng \
%endif
        --disable-static \
        --enable-shared \
        --enable-ssl \
        --with-pkidir=%{_sharedstatedir}/openvswitch/pki \
        --enable-usdt-probes \
%if %{with afxdp}
        --enable-afxdp
%else
        --disable-afxdp
%endif
%make_build
popd

%if %{with dpdk}
pushd build-dpdk
ln -s ../configure
%configure \
%if %{with libcapng}
        --enable-libcapng \
%else
        --disable-libcapng \
%endif
        --disable-static \
        --enable-shared \
        --enable-ssl \
        --enable-usdt-probes \
        --with-dpdk=shared \
        --with-pkidir=%{_sharedstatedir}/openvswitch/pki \
        --libdir=%{_libdir}/openvswitch-dpdk \
        --program-suffix=.dpdk \
%if %{with afxdp}
        --enable-afxdp
%else
        --disable-afxdp
%endif
%make_build
popd
%endif

/usr/bin/python3 build-aux/dpdkstrip.py \
        --dpdk \
        < rhel/usr_lib_systemd_system_ovs-vswitchd.service.in \
        > rhel/usr_lib_systemd_system_ovs-vswitchd.service


%install
rm -rf %{buildroot}

%if %{with dpdk}
make -C build-dpdk install-exec DESTDIR=%{buildroot}

# only need ovs-vswitchd-dpdk and some libraries for dpdk subpackage
rm -rf %{buildroot}%{_bindir}
find %{buildroot}%{_sbindir} -mindepth 1 -maxdepth 1 -not -name ovs-vswitchd.dpdk -delete
find %{buildroot}%{_libdir}/openvswitch-dpdk -mindepth 1 -maxdepth 1 -not -name "libofproto*.so.*" -not -name "libopenvswitch*.so.*" -delete
%endif

%make_install -C build
mv %{buildroot}%{_sbindir}/ovs-vswitchd %{buildroot}%{_sbindir}/ovs-vswitchd.nodpdk
touch %{buildroot}%{_sbindir}/ovs-vswitchd

install -d -m 0755 %{buildroot}/run/openvswitch
install -d -m 0750 %{buildroot}%{_localstatedir}/log/openvswitch
install -d -m 0755 %{buildroot}%{_sysconfdir}/openvswitch

install -p -D -m 0644 %{SOURCE1} %{buildroot}%{_sysusersdir}/openvswitch.conf

install -p -D -m 0644 rhel/usr_lib_udev_rules.d_91-vfio.rules \
        %{buildroot}%{_udevrulesdir}/91-vfio.rules

install -p -D -m 0644 \
        rhel/usr_share_openvswitch_scripts_systemd_sysconfig.template \
        %{buildroot}/%{_sysconfdir}/sysconfig/openvswitch

for service in openvswitch ovsdb-server ovs-vswitchd ovs-delete-transient-ports \
               openvswitch-ipsec; do
        install -p -D -m 0644 \
                        rhel/usr_lib_systemd_system_${service}.service \
                        %{buildroot}%{_unitdir}/${service}.service
done

install -m 0755 rhel/etc_init.d_openvswitch \
        %{buildroot}%{_datadir}/openvswitch/scripts/openvswitch.init

install -p -D -m 0644 rhel/etc_openvswitch_default.conf \
        %{buildroot}/%{_sysconfdir}/openvswitch/default.conf

install -p -D -m 0644 rhel/etc_logrotate.d_openvswitch \
        %{buildroot}/%{_sysconfdir}/logrotate.d/openvswitch

install -m 0644 vswitchd/vswitch.ovsschema \
        %{buildroot}/%{_datadir}/openvswitch/vswitch.ovsschema

install -d -m 0755 %{buildroot}/%{_sysconfdir}/sysconfig/network-scripts/
install -p -m 0755 rhel/etc_sysconfig_network-scripts_ifdown-ovs \
        %{buildroot}/%{_sysconfdir}/sysconfig/network-scripts/ifdown-ovs
install -p -m 0755 rhel/etc_sysconfig_network-scripts_ifup-ovs \
        %{buildroot}/%{_sysconfdir}/sysconfig/network-scripts/ifup-ovs
 
install -d -m 0755 %{buildroot}%{python3_sitelib}
cp -a %{buildroot}/%{_datadir}/openvswitch/python/ovstest \
        %{buildroot}%{python3_sitelib}

pushd python
(
export CPPFLAGS="-I ../include"
export LDFLAGS="%{__global_ldflags} -L %{buildroot}%{_libdir}"
%py3_build
%py3_install
[ -f "%{buildroot}/%{python3_sitearch}/ovs/_json$(python3-config --extension-suffix)" ]
)
popd

rm -rf %{buildroot}/%{_datadir}/openvswitch/python/

install -d -m 0755 %{buildroot}/%{_sharedstatedir}/openvswitch

install -d -m 0755 %{buildroot}%{_prefix}/lib/firewalld/services/

install -p -D -m 0755 \
        rhel/usr_share_openvswitch_scripts_ovs-systemd-reload \
        %{buildroot}%{_datadir}/openvswitch/scripts/ovs-systemd-reload

touch %{buildroot}%{_sysconfdir}/openvswitch/conf.db
# The db needs special permission as IPsec Pre-shared keys are stored in it.
chmod 0640 %{buildroot}%{_sysconfdir}/openvswitch/conf.db
touch %{buildroot}%{_sysconfdir}/openvswitch/system-id.conf

# clean unpackaged files
rm -f %{buildroot}/%{_bindir}/ovs-benchmark \
        %{buildroot}/%{_bindir}/ovs-docker \
        %{buildroot}/%{_bindir}/ovs-parse-backtrace \
        %{buildroot}/%{_sbindir}/ovs-vlan-bug-workaround

rm -rf %{buildroot}/%{_mandir}/*

rm -f %{buildroot}%{_bindir}/ovn*
rm -f %{buildroot}%{_datadir}/openvswitch/ovn*
rm -f %{buildroot}%{_datadir}/openvswitch/scripts/ovn*
rm -f %{buildroot}%{_includedir}/ovn/*

%check
for dir in build \
%if %{with dpdk}
%ifarch %{dpdkarches}
build-dpdk \
%endif
%endif
; do
pushd $dir
%if %{with check}
    touch resolv.conf
    export OVS_RESOLV_CONF=$(pwd)/resolv.conf
    if make check TESTSUITEFLAGS='%{_smp_mflags}' ||
       make check TESTSUITEFLAGS='--recheck' ||
       make check TESTSUITEFLAGS='--recheck'; then :;
    else
        cat tests/testsuite.log
        exit 1
    fi
%endif
%if %{with check_datapath_kernel}
    if make check-kernel RECHECK=yes; then :;
    else
        cat tests/system-kmod-testsuite.log
        exit 1
    fi
%endif
popd
done
 
%preun
%if 0%{?systemd_preun:1}
    %systemd_preun %{name}.service
%else
    if [ $1 -eq 0 ] ; then
    # package removal, not upgrade
        /bin/systemctl --no-reload disable %{name}.service >/dev/null 2>&1 || :
        /bin/systemctl stop %{name}.service >/dev/null 2>&1 || :
    fi
%endif

%pre
%sysusers_create_compat %{SOURCE1}
[ -L %{_sbindir}/ovs-vswitchd ] || rm -f %{_sbindir}/ovs-vswitchd

%post
%{_sbindir}/update-alternatives --install %{_sbindir}/ovs-vswitchd \
  ovs-vswitchd %{_sbindir}/ovs-vswitchd.nodpdk 10
if [ $1 -eq 1 ]; then
    sed -i 's:^#OVS_USER_ID=:OVS_USER_ID=:' /etc/sysconfig/openvswitch

    sed -i \
        's@OVS_USER_ID="openvswitch:openvswitch"@OVS_USER_ID="root:root"@'\
        /etc/sysconfig/openvswitch
fi
chown -R openvswitch:openvswitch /etc/openvswitch
sed -i '/^\[Service\]/ a\Slice=system-ovs.slice' /usr/lib/systemd/system/ovs-vswitchd.service

%if 0%{?systemd_post:1}
    %systemd_post %{name}.service
%else
    # package install, not upgrade
    if [ $1 -eq 1 ]; then
        /bin/systemctl daemon-reload >dev/null || :
    fi
%endif

%postun
if [ $1 -eq 0 ] ; then
  %{_sbindir}/update-alternatives --remove ovs-vswitchd %{_sbindir}/ovs-vswitchd.nodpdk
fi
%if 0%{?systemd_postun:1}
    %systemd_postun %{name}.service
%else
    /bin/systemctl daemon-reload >/dev/null 2>&1 || :
%endif

%if %{with dpdk}
%post dpdk
if grep -Fqw sse4_1 /proc/cpuinfo; then
    priority=20
else
    echo "Warning: the CPU doesn't support SSE 4.1, dpdk support is not enabled." >&2
    priority=5
fi
%{_sbindir}/update-alternatives --install %{_sbindir}/ovs-vswitchd \
  ovs-vswitchd %{_sbindir}/ovs-vswitchd.dpdk $priority

%postun dpdk
if [ $1 -eq 0 ] ; then
  %{_sbindir}/update-alternatives --remove ovs-vswitchd %{_sbindir}/ovs-vswitchd.dpdk
fi
%endif

%files -n python3-openvswitch
%license LICENSE
%{_bindir}/ovs-dpctl-top
%{_sbindir}/ovs-bugtool
%{python3_sitearch}/ovs
%{python3_sitearch}/ovs-*.egg-info
%{_datadir}/openvswitch/bugtool-plugins/
%{_datadir}/openvswitch/scripts/ovs-bugtool-*
%{_datadir}/openvswitch/scripts/ovs-check-dead-ifs
%{_datadir}/openvswitch/scripts/ovs-vtep

%files test
%{_bindir}/ovs-pcap
%{_bindir}/ovs-tcpdump
%{_bindir}/ovs-tcpundump
%{_bindir}/ovs-test
%{_bindir}/ovs-vlan-test
%{_bindir}/ovs-l3ping
%{_datadir}/openvswitch/scripts/usdt/*
%{python3_sitelib}/ovstest

%files testcontroller
%{_bindir}/ovs-testcontroller

%files devel
%{_libdir}/*.so
%{_libdir}/pkgconfig/*.pc
%{_includedir}/openvswitch/*
%{_includedir}/openflow/*
%exclude %{_libdir}/*.a
%exclude %{_libdir}/*.la

%files -n network-scripts-%{name}
%{_sysconfdir}/sysconfig/network-scripts/ifup-ovs
%{_sysconfdir}/sysconfig/network-scripts/ifdown-ovs

%files ipsec
%{_datadir}/openvswitch/scripts/ovs-monitor-ipsec
%{_unitdir}/openvswitch-ipsec.service

%if %{with dpdk}
%files dpdk
%{_libdir}/openvswitch-dpdk/
%ghost %{_sbindir}/ovs-vswitchd
%{_sbindir}/ovs-vswitchd.dpdk
%endif

%files
%license LICENSE
%doc NOTICE README.rst NEWS 
%defattr(-,openvswitch,openvswitch)
%dir %{_sysconfdir}/openvswitch
%{_sysconfdir}/openvswitch/default.conf
%config %ghost %verify(not owner group md5 size mtime) %{_sysconfdir}/openvswitch/conf.db
%ghost %attr(0600,-,-) %verify(not owner group md5 size mtime) %{_sysconfdir}/openvswitch/.conf.db.~lock~
%config %ghost %{_sysconfdir}/openvswitch/system-id.conf
%defattr(-,root,root)
%config(noreplace) %verify(not md5 size mtime) %{_sysconfdir}/sysconfig/openvswitch
%config(noreplace) %{_sysconfdir}/logrotate.d/openvswitch
%{_sysconfdir}/bash_completion.d/ovs-appctl-bashcomp.bash
%{_sysconfdir}/bash_completion.d/ovs-vsctl-bashcomp.bash
%{_sysconfdir}/sysconfig/network-scripts/ifup-ovs
%{_sysconfdir}/sysconfig/network-scripts/ifdown-ovs
%{_sysusersdir}/openvswitch.conf
%{_bindir}/ovs-appctl
%{_bindir}/ovs-dpctl
%{_bindir}/ovs-ofctl
%{_bindir}/ovs-vsctl
%{_bindir}/ovsdb-client
%{_bindir}/ovsdb-tool
%{_bindir}/ovs-pki
%{_bindir}/vtep-ctl
%ghost %{_sbindir}/ovs-vswitchd
%{_sbindir}/ovs-vswitchd.nodpdk
%{_sbindir}/ovsdb-server
%{_libdir}/*.so.*
%{_udevrulesdir}/91-vfio.rules
%{_unitdir}/openvswitch.service
%{_unitdir}/ovsdb-server.service
%{_unitdir}/ovs-vswitchd.service
%{_unitdir}/ovs-delete-transient-ports.service
%{_datadir}/openvswitch/scripts/openvswitch.init
%{_datadir}/openvswitch/scripts/ovs-lib
%{_datadir}/openvswitch/scripts/ovs-save
%{_datadir}/openvswitch/scripts/ovs-ctl
%{_datadir}/openvswitch/scripts/ovs-kmod-ctl
%{_datadir}/openvswitch/scripts/ovs-systemd-reload
%config %{_datadir}/openvswitch/local-config.ovsschema
%config %{_datadir}/openvswitch/vswitch.ovsschema
%config %{_datadir}/openvswitch/vtep.ovsschema
/var/lib/openvswitch
%attr(750,openvswitch,openvswitch) %verify(not owner group) /var/log/openvswitch
%ghost %attr(755,root,root) %verify(not owner group) /run/openvswitch

%changelog
* Sun Feb 18 2024 jackeyji <jackeyji@tencent.com> - 3.1.1-5
- fix CVE-2023-3966

* Tue Sep 19 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.1.1-4
- Rebuilt for python 3.11

* Mon Sep 18 2023 Miaojun Dong <zoedong@tencent.com> - 3.1.1-3
- Rebuild for pyOpenSSL-23.2.0

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.1.1-2
- Rebuilt for OpenCloudOS Stream 23.09

* Wed Aug 9 2023 Wang Guodong <gordonwwang@tencent.com> - 3.1.1-1
- Upgrade to 3.1.1 for dpdk-22.11.1
- fix warnings in versions above groff-1.23 

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 2.17.5-2
- Rebuilt for OpenCloudOS Stream 23.05

* Wed Apr 19 2023 Wang Guodong <gordonwwang@tencent.com> - 2.17.5-1
- initial build
